@@ -3,7 +3,7 @@
 ;; Copyright (C) 2006-2009, 2011-2012, 2015-2018
 ;;   Phil Hagelberg, Doug Alcorn, Will Farrington, Chen Bin
 ;;
-;; Version: 5.7.7
+;; Version: 5.7.9
 ;; Author: Phil Hagelberg, Doug Alcorn, and Will Farrington
 ;; Maintainer: Chen Bin <chenbin.sh@gmail.com>
 ;; URL: https://github.com/technomancy/find-file-in-project
@@ -47,8 +47,10 @@
 ;;   - `find-directory-in-project-by-selected' uses the select region
 ;;      to find directory.  You can provide the keyword if no region
 ;;      is selected.
-;;   - `find-file-in-project' will start search file immediately
+;;   - `find-file-in-project' starts search file immediately
 ;;   - `ffip-create-project-file' creates ".dir-locals.el"
+;;   - `ffip-lisp-find-file-in-project' finds file in project.
+;;     If its parameter is not nil, it find directory.
 ;;
 ;; A project is found by searching up the directory tree until a file
 ;; is found that matches `ffip-project-file'.
@@ -150,6 +152,7 @@
 
 ;;; Code:
 
+(require 'find-lisp)
 (require 'diff-mode)
 (require 'windmove)
 (require 'subr-x)
@@ -157,10 +160,10 @@
 (defvar ffip-use-rust-fd nil "Use use fd instead of find.")
 
 (defvar ffip-rust-fd-respect-ignore-files t
-  "Don 't show search results from '.*ignore' files")
+  "Don 't show search results from '.*ignore' files.")
 
 (defvar ffip-rust-fd-extra-opts ""
-  "rust fd extra options passed to cli.")
+  "Rust fd extra options passed to cli.")
 
 (defvar ffip-window-ratio-alist
   '((1 . 1.61803398875)
@@ -201,9 +204,9 @@ The file path is passed to the hook as the first argument.")
         standard-output
       (shell-command command t))))
 
-(defun ffip-nonempty-lines (s)
-  "Return non empty lines."
-  (split-string s "[\r\n]+" t))
+(defun ffip-nonempty-lines (str)
+  "Return non empty lines from STR."
+  (split-string str "[\r\n]+" t))
 
 (defun ffip-diff-git-versions ()
   "List all versions of code under Git."
@@ -360,10 +363,10 @@ May be set using .dir-locals.el.  Checks each entry if set to a list.")
 Use this to exclude portions of your project: \"-not -regex \\\".*svn.*\\\"\".")
 
 (defvar ffip-find-pre-path-options ""
-  "Extra options to pass to `find' before path name options when using `find-file-in-project'.
+  "Options for find porgram.
 
-As required by `find', `-H', `-L', `-P', `-D' and `-O' must appear before the first path name, `.'.
-For example, use this to follow symbolic links inside your project: \"-L\".")
+GNU Find requires '-H', '-L', '-P', '-D' and `-O' appear before first path '.'.
+For example, use '-L' to follow symbolic links.")
 
 (defvar ffip-project-root nil
   "If non-nil, overrides the project root directory location.")
@@ -403,7 +406,7 @@ This overrides variable `ffip-project-root' when set.")
   "The callback after calling `find-relative-path'.")
 
 (defun ffip--some (predicate seq)
-  "Return if PREDICATE is of true of any element of SEQ"
+  "Return if PREDICATE is t for any element of SEQ."
   (let* (elem rlt)
     (while (and (setq elem (car seq))
                 (not rlt))
@@ -413,7 +416,7 @@ This overrides variable `ffip-project-root' when set.")
 
 ;;;###autoload
 (defun ffip-project-root ()
-  "Return the root of the project."
+  "Return project root or `default-directory'."
   (let* ((project-root (or ffip-project-root
                            (cond
                             ((functionp ffip-project-root-function)
@@ -426,9 +429,7 @@ This overrides variable `ffip-project-root' when set.")
                              (locate-dominating-file default-directory
                                                      ffip-project-file))))))
     (or (and project-root (file-name-as-directory project-root))
-        (progn
-          (message "Since NO project was found, use `default-directory' instead.")
-          default-directory))))
+        default-directory)))
 
 (defun ffip--read-file-text (file)
   "Read text from FILE."
@@ -771,12 +772,13 @@ This function is the API to find files."
   (cons 'ffip-project-root root))
 
 (defun ffip--read-selected ()
+  "Read select string."
   (buffer-substring-no-properties (region-beginning) (region-end)))
 
 (defun ffip-read-keyword ()
   "Read keyword from selected text or user input."
-  (let* ((hint (if ffip-use-rust-fd "Enter regex (or press ENTER):"
-                 "Enter keyword (or press ENTER):"))
+  (let* ((hint (if ffip-use-rust-fd "Enter regex (or press ENTER): "
+                 "Enter keyword (or press ENTER): "))
          rlt)
     (cond
      ((region-active-p)
@@ -853,15 +855,19 @@ You can override this by setting the variable `ffip-project-root'."
   "Is FILENAME relative?"
   (if (string-match-p ffip-relative-path-pattern filename) t))
 
+(defun ffip-guess-file-name-at-point ()
+  "Guess file name at point."
+  (or (and (region-active-p) (ffip--read-selected))
+      (thing-at-point 'filename)
+      (thing-at-point 'symbol)
+      (read-string "No file name at point. Please provide one:")))
+
 ;;;###autoload
 (defun find-file-in-project-at-point (&optional open-another-window)
   "Find file whose name is guessed around point.
 If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
   (interactive "P")
-  (let* ((fn (or (and (region-active-p) (ffip--read-selected))
-                       (thing-at-point 'filename)
-                       (thing-at-point 'symbol)
-                       (read-string "No file name at point. Please provide file name:")))
+  (let* ((fn (ffip-guess-file-name-at-point))
          ;; could be a path
          (ffip-match-path-instead-of-filename t)
          tfn)
@@ -969,11 +975,11 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
     (find-file-in-project-by-selected open-another-window)))
 
 ;;;###autoload
-(defun find-relative-path(&optional find-directory)
+(defun ffip-find-relative-path(&optional find-directory)
   "Find file/directory and copy its relative path into `kill-ring'.
-Optional prefix FIND-DIRECTORY copy the directory path; file path by default.
+If FIND-DIRECTORY is t, copy the directory path.
 
-You can set `ffip-find-relative-path-callback' to format the string before copying,
+Set `ffip-find-relative-path-callback' to format the result,
   (setq ffip-find-relative-path-callback 'ffip-copy-reactjs-import)
   (setq ffip-find-relative-path-callback 'ffip-copy-org-file-link)"
   (interactive "P")
@@ -1011,9 +1017,50 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
   (interactive "P")
   (ffip-find-files (ffip-read-keyword) open-another-window t))
 
+(defun ffip--prune-patterns-regex ()
+  "Convert `ffip--prune-patterns-regex to regex."
+  (let* ((rlt (mapconcat 'identity ffip-prune-patterns "\\|")))
+    (setq rlt (replace-regexp-in-string "\\." "\\\\." rlt))
+    (setq rlt (replace-regexp-in-string "\\*" ".*" rlt))
+    rlt))
+
+;;;###autoload
+(defun ffip-lisp-find-file-in-project (&optional directory-p)
+  "If DIRECTORY-P is nil, find file in project, or else find directory.
+It's is written in pure Lisp, so it works in all environments."
+  (interactive "P")
+  (let* ((root (ffip-get-project-root-directory))
+         (input-regex (read-string "Input regex (or press ENTER): "))
+         (find-lisp-regexp (if (string= input-regex "") ".*" input-regex))
+         cands
+         (ignored-regex (ffip--prune-patterns-regex)))
+    (cond
+     (directory-p
+      (setq cands (find-lisp-find-files-internal
+                  root
+                  'find-lisp-file-predicate-is-directory
+                  'find-lisp-default-directory-predicate)))
+     (t
+      (setq cands (find-lisp-find-files-internal
+                   root
+                   'find-lisp-default-file-predicate
+                   'find-lisp-default-directory-predicate))))
+    (setq cands
+          (delq nil
+                (mapcar `(lambda (c)
+                           (unless (string-match-p ,ignored-regex c) c))
+                        cands)))
+    (ffip-completing-read
+     (format "%s %s: " (if directory-p "directories" "files") root)
+     cands
+     `(lambda (item)
+        (if ,directory-p
+            (switch-to-buffer (dired item))
+          (find-file item))))))
+
 ;;;###autoload
 (defalias 'ffip 'find-file-in-project)
-
+(defalias 'find-relative-path 'ffip-find-relative-path)
 
 (defun ffip-path (candidate)
   "Get path from ivy CANDIDATE."
@@ -1084,8 +1131,11 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
         (setq blnum (string-to-number (match-string 3)))))
 
     (cond
-     ((and (> (length files) 1)
-           (string= (nth 0 files) (nth 1 files)))
+     ((or (null files) (eq (length files) 0))
+      (message "No file is found!"))
+     ((or (and (> (length files) 1)
+               (string= (nth 0 files) (nth 1 files)))
+          (eq (length files) 1))
       (ffip-find-files (nth 0 files)
                        open-another-window
                        nil
@@ -1094,8 +1144,13 @@ If OPEN-ANOTHER-WINDOW is not nil, the file will be opened in new window."
                           ;; is only one file name candidate
                           (ffip--forward-line ,blnum))))
      (t
+      ;; need pick a file
       (run-hook-with-args 'ffip-diff-find-file-before-hook)
-      (ffip-find-files files
+      (ffip-find-files (cond
+                        ((string= (nth 0 files) "null")
+                         (nth 1 files))
+                        (t
+                         (nth 0 files)))
                        open-another-window
                        nil
                        (lambda (opened-file)
@@ -1220,6 +1275,7 @@ If NUM is not nil, the corresponding backend is executed directly."
 (defalias 'ffip-show-diff 'ffip-show-diff-by-description)
 
 (defadvice read-file-name (around ffip-read-file-name-hack activate)
+  "Advice `read-file-name'."
   (cond
    (ffip-read-file-name-hijacked-p
     ;; only hack read-file-name once
